#include <iostream>
#include <fstream>
#include <sys/ioctl.h>
#include <stdio.h>
#include <math.h>
#include <unistd.h>
#include <fcntl.h>
#include <sys/types.h>
#include <errno.h>
#include <linux/soundcard.h>
#include <assert.h>
#include <sys/time.h>

using namespace std;

struct timeval tmp, tmp2;
struct timezone tzp;

void dual_tone(int audio_fd, int samples, double freq1, double freq2) {
  ofstream fdes("stime.txt", ios::app);
  static double Pi = 3.14159;
  unsigned char *buffer;
  double sampleperiod = 1.0/8000.0;
  int sample;

  freq1 *= 2*Pi;
  freq2 *= 2*Pi;

  buffer = new unsigned char[samples];
  
  
  for (sample=0; sample<samples; sample++) 
      buffer[sample] = (unsigned char)(128.0 + 
	                  40*( sqrt(1 - sin(freq1*sample*sampleperiod)) ) +
	                  40*( sqrt(1 - sin(freq2*sample*sampleperiod)) ));
  
  
  gettimeofday(&tmp, &tzp);
  write(audio_fd, buffer, samples*sizeof(unsigned char));
  //gettimeofday(&tmp/*2*/, &tzp);

  cout << "\nAudio signal transmitted at :\n";
  cout << tmp.tv_sec << "\t";
  fdes << tmp.tv_sec << endl;
  fdes << tmp.tv_usec << endl;
  cout << tmp.tv_usec << "\n";

  /*cout<< "\nDiff :\n";
  cout<< tmp2.tv_sec << "\n";
  cout << tmp2.tv_usec - tmp.tv_usec << "\n";*/
  fdes << flush;
  fdes.close();

  delete buffer;
}

void play_dtmf(int audio_fd, int key, int duration) {
  char buffer[500];
  int s;
/* TEN FOLD */
  double col_freq[3] = { 12090.0, 13360.0, 14770.0 };
  double row_freq[4] = { 6970.0, 7700.0, 8520.0, 9410.0 };
  int row, col;

  if (key) {
    row = (key - 1) / 3;
    col = (key - 1) % 3;
  }
  else {     /* Key is 0 */
    row = 3;
    col = 1;
  }
 
 // printf(" key %u row %u col %u freqs %f %f \n",key,row,col,row_freq[row],col_freq[col]);

  dual_tone(audio_fd,duration,row_freq[row],col_freq[col]);

  /* Play a short bit of silence */

  for (s = 0; s<500; s++) buffer[s] = 128;
  write(audio_fd,buffer,100);
}

int main(int argc, char **argv) {
  int audio_fd;                /* Will hold the file handle of the audiodev */
  char *device = "/dev/audio";   /* The filename of the audio device */
  int format;	               /* General purpose variable */
  
  
  char *number = "9999999999";    /* Phone number to dial. */

  /* Open the audio device */

  if ((audio_fd = open(device,O_WRONLY,0)) == -1) {
      fprintf(stderr,"Couldn't open audio device. (%s)\n",strerror(errno));
      exit(1);
  }

  /* Set the sample format to unsigned, 8 bits/sample */

  format = AFMT_U8;
  if (ioctl(audio_fd, SNDCTL_DSP_SETFMT, &format) == -1) {
      printf("Couldn't change audio format.\n");
      exit(1);
  }

  /* Single channel mode -- disable stereo */
  format = 0;
  assert( ioctl(audio_fd, SNDCTL_DSP_STEREO, &format) != -1);

  /* Set the audio device to 8000 samples per second */
  format = 48000;
  assert( ioctl(audio_fd, SNDCTL_DSP_SPEED, &format) != -1);

  /* Play each digit of the number in turn. */
//  dual_tone(audio_fd,16000,350.0,440.0);
  while (*number) {
   play_dtmf(audio_fd,(*number)-'0',960);
   number++;
   sleep(5);
  }
//  dual_tone(audio_fd,4000,440.0,480.0);
  /* Close the audio device. */

  close(audio_fd);

  return (0);
}
